@@ -1,6 +1,6 @@
+"""Interface for ``python -m scanspec``."""
+
 from scanspec import cli
 
-# test with:
-#     pipenv run python -m scanspec
 if __name__ == "__main__":
     cli.cli()
