@@ -1,3 +1,5 @@
+"""Interface for ``python -m scanspec``."""
+
 import logging
 import string
 
@@ -16,15 +18,18 @@ from .specs import *  # noqa
         ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"], case_sensitive=False
     ),
 )
-@click.version_option(prog_name="scanspec")
+@click.version_option(prog_name="scanspec", message="%(version)s")
 @click.pass_context
-def cli(ctx, log_level: str):
+def cli(ctx: click.Context, log_level: str):
     """Top level scanspec command line interface."""
     level = getattr(logging, log_level.upper(), None)
     logging.basicConfig(format="%(levelname)s:%(message)s", level=level)
 
     # if no command is supplied, print the help message
     if ctx.invoked_subcommand is None:
+        # We need to prove that cli has been converted to a command
+        # by the click decorator to keep pyright happy.
+        assert isinstance(cli, click.Command)
         click.echo(cli.get_help(ctx))
 
 
@@ -45,8 +50,8 @@ def plot(spec: str):
 @click.option(
     "--port", default=8080, help="The port that the scanspec service will be hosted on."
 )
-def service(cors, port):
-    """Run up a GraphQL service."""
+def service(cors: bool, port: int):
+    """Run up a REST service."""
     from scanspec.service import run_app
 
     run_app(cors, port)
@@ -54,7 +59,7 @@ def service(cors, port):
 
 @cli.command()
 def schema():
-    """Print the GraphQL schema for the service."""
+    """Print the OpenAPI schema for the service."""
     from scanspec.service import scanspec_schema_text
 
     click.echo(scanspec_schema_text())
