@@ -1,3 +1,5 @@
+from typing import Any
+
 import pytest
 
 from scanspec.regions import Region
@@ -8,13 +10,14 @@ def test_not_implemented() -> None:
     with pytest.raises(NotImplementedError):
         Region().axis_sets()
     with pytest.raises(NotImplementedError):
-        Region().mask({})
+        region: Region[Any] = Region()
+        region.mask({})
     with pytest.raises(NotImplementedError):
         Spec().axes()
     with pytest.raises(NotImplementedError):
         Spec().calculate()
     with pytest.raises(TypeError):
-        Spec() * Region()
+        Spec() * Region()  # type: ignore
 
 
 def test_non_snake_not_allowed_inside_snaking_dim() -> None:
